// ----------------------------------------------------------------------------
//
//  Copyright (C) 2020 Fons Adriaensen <fons@linuxaudio.org>
//    
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// ----------------------------------------------------------------------------


//-----------------------------------------------------------------------------------
// Common definitions
//-----------------------------------------------------------------------------------


#include "dominance1.h"
#include "sn3dpan1.h"
#include "zrotator1.h"


static const char* author = "Fons Adriaensen <fons@linuxaudio.org>";
static const char* license = "GPL";


static void pconnect (LADSPA_Handle H, unsigned long port, LADSPA_Data *data)
{
    ((LadspaPlugin *)H)->setport (port, data);
}

static void activate (LADSPA_Handle H)
{
    ((LadspaPlugin *)H)->active (true);
}

static void runplugin (LADSPA_Handle H, unsigned long k)
{
    ((LadspaPlugin *)H)->runproc (k, false);
}
/*
  static void runadding (LADSPA_Handle H, unsigned long k)
  {
  ((LadspaPlugin *)H)->runproc (k, true);
  }

  static void setadding (LADSPA_Handle H, LADSPA_Data gain)
  {
  ((LadspaPlugin *)H)->setgain (gain);
  }
*/
static void deactivate (LADSPA_Handle H)
{
    ((LadspaPlugin *)H)->active (false);
}

static void cleanup (LADSPA_Handle H)
{
    delete (LadspaPlugin *) H;
}


//-----------------------------------------------------------------------------------
//  LADSPA dlsym interface
//-----------------------------------------------------------------------------------


#define NMODS 4
#define VERSION "0.3.0"


static const LADSPA_Descriptor moddescr [NMODS] =
{
    {
	3711,
        "dominance1",
	LADSPA_PROPERTY_HARD_RT_CAPABLE,
        "Dominance1 (SN3D)",
	author,
	license,
	Ladspa_Dominance1::NPORT,
	Ladspa_Dominance1::portdescr,
	Ladspa_Dominance1::portnames,
	Ladspa_Dominance1::porthints,
	0,
	Ladspa_Dominance1::create,
	pconnect,
	activate,
	runplugin,
	0,
	0,
	deactivate,
	cleanup
    },  
    {
	3714,
	"SN3D-panner1",
	LADSPA_PROPERTY_HARD_RT_CAPABLE,
        "Degree 1 AMB panner (Mono -> SN3D)",
	author,
	license,
	Ladspa_Sn3dpan1::NPORT,
	Ladspa_Sn3dpan1::portdescr,
	Ladspa_Sn3dpan1::portnames,
	Ladspa_Sn3dpan1::porthints,
	0,
	Ladspa_Sn3dpan1::create,
	pconnect,
	activate,
	runplugin,
	0,
	0,
	deactivate,
	cleanup
    },  
    {
	3715,
        "SN3D-panner1s",
	LADSPA_PROPERTY_HARD_RT_CAPABLE,
        "Degree 1 AMB panner (Stereo > SN3D)",
	author,
	license,
	Ladspa_Sn3dpan1s::NPORT,
	Ladspa_Sn3dpan1s::portdescr,
	Ladspa_Sn3dpan1s::portnames,
	Ladspa_Sn3dpan1s::porthints,
	0,
	Ladspa_Sn3dpan1s::create,
	pconnect,
	activate,
	runplugin,
	0,
	0,
	deactivate,
	cleanup
    },
    {
	3716,
        "AMB-Z-rotator1",
	LADSPA_PROPERTY_HARD_RT_CAPABLE,
        "Degree 1 AMB Z-rotator",
	author,
	license,
	Ladspa_Zrotator1::NPORT,
	Ladspa_Zrotator1::portdescr,
	Ladspa_Zrotator1::portnames,
	Ladspa_Zrotator1::porthints,
	0,
	Ladspa_Zrotator1::create,
	pconnect,
	activate,
	runplugin,
	0,
	0,
	deactivate,
	cleanup
    }  
};


extern "C" const LADSPA_Descriptor *ladspa_descriptor (unsigned long i)
{
    if (i >= NMODS) return 0;
    return moddescr + i;
}


