// ----------------------------------------------------------------------------
//
//  Copyright (C) 2020 Fons Adriaensen <fons@linuxaudio.org>
//    
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// ----------------------------------------------------------------------------


#include <string.h>
#include <math.h>
#include "zrotator1.h"


#define DEG2RAD 0.01745329f


const LADSPA_PortDescriptor Ladspa_Zrotator1::portdescr [NPORT] = 
{
    LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
    LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
    LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
    LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
    LADSPA_PORT_INPUT  | LADSPA_PORT_CONTROL
};


const char * const Ladspa_Zrotator1::portnames [NPORT] = 
{
    "in.0", "in.1", "in.2", "in.3",
    "out.0", "out.1", "out.2", "out.3",
    "Az"
};


const LADSPA_PortRangeHint Ladspa_Zrotator1::porthints [NPORT] = 
{
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { 0, 0, 0 },
    { LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE | LADSPA_HINT_DEFAULT_0, -180, 180 }
};


LADSPA_Handle Ladspa_Zrotator1::create (const struct _LADSPA_Descriptor *desc, unsigned long rate)
{
    return new Ladspa_Zrotator1 (rate);
}


void Ladspa_Zrotator1::setport (unsigned long port, LADSPA_Data *data)
{
    _port [port] = data;
}


void Ladspa_Zrotator1::active (bool act)
{
    if (act) calcpar (0.0f);
}


void Ladspa_Zrotator1::calcpar (float az)
{
    az *= DEG2RAD;
    _c1 = cosf (az);
    _s1 = sinf (az);
}


void Ladspa_Zrotator1::runproc (unsigned long len, bool add)
{
    int    k;
    float  c1, s1;
    float  x, y, dc, ds;
    float  *ipx, *ipy;
    float  *opx, *opy;

    c1 = _c1;
    s1 = _s1;
    calcpar (_port [CTL_AZ][0]);

    memcpy (_port [OUT +  0], _port [INP +  0], len * sizeof (LADSPA_Data));
    memcpy (_port [OUT +  2], _port [INP +  2], len * sizeof (LADSPA_Data));

    ipx = _port [INP + 3];
    ipy = _port [INP + 1];
    opx = _port [OUT + 3];
    opy = _port [OUT + 1];
    k = len;
    dc = (_c1 - c1) / len;
    ds = (_s1 - s1) / len;
    while (k--)
    {
	c1 += dc;
	s1 += ds;
        x = *ipx++;
        y = *ipy++; 
        *opx++ = c1 * x + s1 * y;
        *opy++ = c1 * y - s1 * x;
    }
}
